//! Most straight forward smart pointer is a box, `Box<t>`.
//! Use boxes most often when:
//! 1) When have type whose size not known at compile time
//!    and want to use value of said type in a context
//!    that requires an exact size
//! 2) When have large amount of data and want to transfer
//!    ownership but ensure data won't be copied
//! 3) When want to own a value and only care it implements
//!    a particular trait rather than being of a specific type

use crate::List::{Cons, Nil};

enum List {
    // Because Box<T> is a pointer, Rust knows how much
    // space a Box<T> needs.
    Cons(i32, Box<List>),
    Nil,
}

fn main() {
    let b = Box::new(5);
    println!("b = {b}");

    // enabling recursive types with Boxes
    // Recursive types an issue because Rust needs to know
    // how much space a type takes up. Boxes have a known size
    // so can enable recursive types by inserting a box in
    // the recursive type definition. Basically Haskell.
    // That's it, time to rewrite the ghc in Rust.
    let list = Cons(1, Box::new(Cons(2, Box::new(Cons(3, Box::new(Nil))))));
}
